import React, { useRef, useEffect, useCallback, useState } from 'react';
import classnames from 'classnames';
import { useScrollObserver } from '@highlight-ui/utils-hooks';
import { parseMetadata } from '@highlight-ui/utils-commons';
import { Typography } from '@highlight-ui/typography';
import { Button } from '@highlight-ui/button';
import { Icon } from '@highlight-ui/icon';
import { GroupedPortal } from '@highlight-ui/utils-portal-manager';
import noop from 'lodash/noop';
import invoke from 'lodash/invoke';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

var styles$1 = {"dialog":"Dialog-module__2icvnh15__v2-0-14","size-small":"Dialog-module__3spVEVua__v2-0-14","size-medium":"Dialog-module__-zWxw4HX__v2-0-14","size-large":"Dialog-module__3wZ2NPV0__v2-0-14","header":"Dialog-module__2wAUH40a__v2-0-14","headerShadowed":"Dialog-module__xODFOi1M__v2-0-14","headerTitle":"Dialog-module__1lxoaZF6__v2-0-14","closeButton":"Dialog-module__2l9SSdo___v2-0-14","body":"Dialog-module__1bPY4avA__v2-0-14","footer":"Dialog-module__3o8NTS6m__v2-0-14","footerShadowed":"Dialog-module__1s03rY8s__v2-0-14","footerAction":"Dialog-module__2YsxojU___v2-0-14"};

function useEventListener(t,r,i,o){void 0===i&&(i=global),void 0===o&&(o={});var c=useRef(),u=o.capture,a=o.passive,v=o.once;useEffect(function(){c.current=r;},[r]),useEffect(function(){if(i&&i.addEventListener){var e=function(e){return c.current(e)},n={capture:u,passive:a,once:v};return i.addEventListener(t,e,n),function(){i.removeEventListener(t,e,n);}}},[t,i,u,a,v]);}

/*!
 * Adapted from jQuery UI core
 *
 * http://jqueryui.com
 *
 * Copyright 2014 jQuery Foundation and other contributors
 * Released under the MIT license.
 * http://jquery.org/license
 *
 * http://api.jqueryui.com/category/ui-core/
 */
var tabbableNode = /input|select|textarea|button|object/;
var focusSelector = 'a, input, select, textarea, button, object, [tabindex]';

function hidden(el) {
  if (process.env.NODE_ENV === 'test') return false;
  return el.offsetWidth <= 0 && el.offsetHeight <= 0 || el.style.display === 'none';
}

function visible(element) {
  var parentElement = element;

  while (parentElement) {
    if (parentElement === document.body) break;
    if (hidden(parentElement)) return false;
    parentElement = parentElement.parentNode;
  }

  return true;
}

function getElementTabIndex(element) {
  var tabIndex = element.getAttribute('tabindex');
  if (tabIndex === null) tabIndex = undefined;
  return parseInt(tabIndex, 10);
}

function focusable(element) {
  var nodeName = element.nodeName.toLowerCase();
  var isTabIndexNotNaN = !isNaN(getElementTabIndex(element));
  var res = // @ts-ignore
  tabbableNode.test(nodeName) && !element.disabled || (element instanceof HTMLAnchorElement ? element.href || isTabIndexNotNaN : isTabIndexNotNaN);
  return res && visible(element);
}
function tabbable(element) {
  var tabIndex = getElementTabIndex(element);
  var isTabIndexNaN = isNaN(tabIndex);
  return (isTabIndexNaN || tabIndex >= 0) && focusable(element);
}
/**
 * Tabbable elements are elements the user can tab between. This excludes elements with tabIndex=-1
 * */

function findTabbableDescendants(element) {
  return Array.from(element.querySelectorAll(focusSelector)).filter(tabbable);
}

var focusLaterElements = [];
var focusElement = null;
var needToFocus = false;

function handleBlur() {
  needToFocus = true;
}

function handleFocus() {
  if (needToFocus) {
    needToFocus = false;

    if (!focusElement) {
      return;
    }

    if (!focusElement) return;

    if (focusElement.contains(document.activeElement)) {
      return;
    }

    var el = findTabbableDescendants(focusElement)[0] || focusElement;
    el.focus();
  }
}

function markForFocusLater() {
  focusLaterElements.push(document.activeElement);
}
function returnFocus() {
  var toFocus = null;

  try {
    toFocus = focusLaterElements.pop();
    if (toFocus) toFocus.focus();
  } catch (e) {
    // eslint-disable-next-line no-console
    console.warn(['You tried to return focus to', toFocus, 'but it is not in the DOM anymore'].join(' '));
  }
}
function setupScopedFocus(element) {
  focusElement = element;
  document.addEventListener('focusout', handleBlur, false);
  document.addEventListener('focusin', handleFocus, true);
}
function teardownScopedFocus() {
  focusElement = null;
  document.removeEventListener('focusout', handleBlur);
  document.removeEventListener('focusin', handleFocus);
}

/**
 * Controls tabbing, so it stays inside the node element
 * @param node
 * @param event
 */

function scopeTab(node, event) {
  var tabbable = findTabbableDescendants(node);

  if (!tabbable.length) {
    event.preventDefault();
    return;
  }

  var finalTabbable = tabbable[event.shiftKey ? 0 : tabbable.length - 1];
  var leavingFinalTabbable = finalTabbable === document.activeElement || // handle immediate shift+tab after opening with mouse
  node === document.activeElement;
  if (!leavingFinalTabbable) return;
  event.preventDefault();
  var target = tabbable[event.shiftKey ? tabbable.length - 1 : 0];
  if (target) target.focus();
}

/**
 * Sets `aria-hidden` on all elements on the body, except for the active node.
 **/
function createAriaHider(containerNode, selector) {
  if (selector === void 0) {
    selector = 'body > :not(script)';
  }

  var rootNodes = Array.from(document.querySelectorAll(selector)).map(function (node) {
    if (node.contains(containerNode)) {
      return undefined;
    }

    var ariaHidden = node.getAttribute('aria-hidden');

    if (ariaHidden === null || ariaHidden === 'false') {
      node.setAttribute('aria-hidden', 'true');
    }

    return {
      node: node,
      ariaHidden: ariaHidden
    };
  });
  return function () {
    rootNodes.forEach(function (item) {
      if (!item) return;

      if (item.ariaHidden === null) {
        item.node.removeAttribute('aria-hidden');
      } else {
        item.node.setAttribute('aria-hidden', item.ariaHidden);
      }
    });
  };
}

/**
 *  Traps focus to DOM node. Use this with a modal to ensure the user focus doesn't leave.
 * */
function useFocusTrap(active, options) {
  if (active === void 0) {
    active = true;
  }

  if (options === void 0) {
    options = {};
  }

  var ref = useRef();
  var restoreAria = useRef(null);
  var setRef = useCallback(function (node) {
    if (restoreAria.current) {
      restoreAria.current();
    }

    if (ref.current) {
      returnFocus();
      teardownScopedFocus();
    }

    if (active && node) {
      setupScopedFocus(node);
      markForFocusLater();

      var processNode = function processNode(node) {
        // See if we should disable aria
        restoreAria.current = !options.disableAriaHider ? createAriaHider(node) : null; // Find the initial focus element

        var focusElement = null;

        if (options.focusSelector) {
          focusElement = typeof options.focusSelector === 'string' ? node.querySelector(options.focusSelector) : options.focusSelector;
        }

        if (!focusElement) {
          var children = Array.from(node.querySelectorAll(focusSelector));
          focusElement = // Prefer tabbable elements, But fallback to any focusable element
          children.find(tabbable) || // But fallback to any focusable element
          children.find(focusable) || // Nothing found
          null; // If everything else fails, see if the node itself can handle focus

          if (!focusElement && focusable(node)) focusElement = node;
        }

        if (focusElement) {
          // Set the initial focus inside the traps
          focusElement.focus();
        } else {
          if (process.env.NODE_ENV === 'development') {
            // eslint-disable-next-line no-console
            console.warn('[useFocusTrap]: Failed to find a focusable element after activating the focus trap. Make sure to include at an element that can recieve focus. As a fallback, you can also set "tabIndex={-1}" on the focus trap node.', node);
          }
        }
      }; // Delay processing the HTML node by a frame. This ensures focus is assigned correctly.


      setTimeout(function () {
        if (node.ownerDocument) {
          processNode(node);
        } else if (process.env.NODE_ENV === 'development') {
          // eslint-disable-next-line no-console
          console.warn('[useFocusTrap]: The focus trap is not part of the DOM yet, so it is unable to correctly set focus. Make sure to render the ref node.', node);
        }
      });
      ref.current = node;
    } else {
      ref.current = null;
    }
  }, [active, options.focusSelector, options.disableAriaHider]);
  useEffect(function () {
    if (!active) return undefined;

    var handleKeyDown = function handleKeyDown(event) {
      if (event.key === 'Tab' && ref.current) {
        scopeTab(ref.current, event);
      }
    };

    document.addEventListener('keydown', handleKeyDown);
    return function () {
      document.removeEventListener('keydown', handleKeyDown);
    };
  }, [active]);
  return setRef;
}

var styles = {"modal":"Modal-module__i5Pbh60m__v2-0-14","modalBackdrop":"Modal-module__ZVIwEMrU__v2-0-14","modalContent":"Modal-module__3UHa_kcB__v2-0-14","scrollLocked":"Modal-module__1T2Qjbem__v2-0-14"};

var Modal = function (_a) {
    var metadata = _a.metadata, children = _a.children, className = _a.className, initialFocusElement = _a.initialFocusElement, onRequestToClose = _a.onRequestToClose, _b = _a.onClose, onClose = _b === void 0 ? noop : _b, _c = _a.onOpen, onOpen = _c === void 0 ? noop : _c, open = _a.open;
    var ref = useFocusTrap(true, {
        disableAriaHider: true,
        focusSelector: initialFocusElement,
    });
    var firstRender = useRef(true);
    useEventListener('keydown', function (event) {
        if (open && event.key === 'Escape')
            onRequestToClose();
    }, window);
    useEffect(function () {
        if (firstRender.current) {
            firstRender.current = false;
            if (!open)
                return;
        }
        if (open) {
            onOpen();
            invoke(document, 'body.classList.add', styles.scrollLocked);
        }
        else {
            onClose();
            invoke(document, 'body.classList.remove', styles.scrollLocked);
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [open]);
    if (!open)
        return null;
    return (React.createElement(GroupedPortal, null,
        React.createElement("div", __assign({}, parseMetadata(metadata), { className: styles.modal, ref: ref }),
            React.createElement("div", { className: styles.modalBackdrop }),
            React.createElement("div", { className: classnames(styles.modalContent, className) }, children))));
};

var DEFAULT_LABEL_ID = 'dialog-title';
var Dialog = function (_a) {
    var _b, _c;
    var metadata = _a.metadata, children = _a.children, className = _a.className, onRequestToClose = _a.onRequestToClose, onClose = _a.onClose, onOpen = _a.onOpen, open = _a.open, title = _a.title, _d = _a.labelledBy, labelledBy = _d === void 0 ? DEFAULT_LABEL_ID : _d, describedBy = _a.describedBy, confirmButton = _a.confirmButton, cancelButton = _a.cancelButton, _e = _a.variant, variant = _e === void 0 ? 'default' : _e, _f = _a.size, size = _f === void 0 ? 'small' : _f;
    var _g = useState(), cancelButtonElement = _g[0], setCancelButtonElement = _g[1];
    var _h = useState(false), topShadow = _h[0], setTopShadow = _h[1];
    var _j = useState(false), bottomShadow = _j[0], setBottomShadow = _j[1];
    var setElement = useScrollObserver(function (percent) {
        setTopShadow(percent > 0);
        setBottomShadow(percent < 100);
    }).setElement;
    return (React.createElement(Modal, { metadata: metadata, className: className, initialFocusElement: cancelButtonElement !== null && cancelButtonElement !== void 0 ? cancelButtonElement : undefined, onClose: onClose, onOpen: onOpen, open: open, onRequestToClose: onRequestToClose },
        React.createElement("section", { role: "dialog", "aria-modal": "true", "aria-labelledby": labelledBy, "aria-describedby": describedBy, className: classnames(styles$1.dialog, styles$1["size-" + size]) },
            React.createElement("header", { className: classnames(styles$1.header, (_b = {},
                    _b[styles$1.headerShadowed] = topShadow,
                    _b)) },
                React.createElement(Typography, { component: "span", variant: "heading-3", className: styles$1.headerTitle, id: labelledBy }, title),
                React.createElement("button", { type: "button", title: "close", onClick: onRequestToClose, className: styles$1.closeButton, "aria-label": "Close Modal" // TODO: This needs to be translated
                 },
                    React.createElement(Icon, { name: "times", type: "solid" }))),
            React.createElement("div", { ref: setElement, className: styles$1.body }, children),
            React.createElement("footer", { className: classnames(styles$1.footer, (_c = {},
                    _c[styles$1.footerShadowed] = bottomShadow,
                    _c)) },
                React.createElement(Button, __assign({ ref: setCancelButtonElement, className: styles$1.footerAction, onClick: cancelButton.onClick, loading: cancelButton.loading, disabled: cancelButton.disabled }, parseMetadata(cancelButton.metadata)), cancelButton.label),
                React.createElement(Button, __assign({ className: classnames(styles$1.footerAction, {
                        'dialog-danger': variant === 'destructive',
                        'dialog-primary': variant !== 'destructive',
                    }), onClick: confirmButton.onClick, loading: confirmButton.loading, disabled: confirmButton.disabled, theme: variant === 'destructive' ? 'danger' : 'primary' }, parseMetadata(confirmButton.metadata)), confirmButton.label)))));
};

export { Dialog, Modal };
//# sourceMappingURL=index.js.map
